@@ -1,8 +1,8 @@
 #! /usr/bin/env/python
 # -*- coding: utf-8 -*-
 
-## Copyright 2008 by LivingLogic AG, Bayreuth/Germany
-## Copyright 2008 by Walter Dörwald
+## Copyright 2009-2010 by LivingLogic AG, Bayreuth/Germany
+## Copyright 2009-2010 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
@@ -13,7 +13,7 @@ import re, datetime, StringIO
 
 import py.test
 
-from ll import ul4c
+from ll import ul4c, color
 
 
 def check(result, source, **variables):
@@ -74,7 +74,7 @@ def checkrunerror(msg, source, **variables):
 	try:
 		t1.renders(**variables)
 	except Exception, exc:
-		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
+		assert re.search(msg, "{0.__class__.__module__}.{0.__class__.__name__}: {0}".format(exc)) is not None
 	else:
 		py.test.fail("Didn't raise exception")
 
@@ -84,7 +84,7 @@ def checkrunerror(msg, source, **variables):
 	try:
 		t2.renders(**variables)
 	except Exception, exc:
-		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
+		assert re.search(msg, "{0.__class__.__module__}.{0.__class__.__name__}: {0}".format(exc)) is not None
 	else:
 		py.test.fail("Didn't raise exception")
 
@@ -97,7 +97,7 @@ def checkrunerror(msg, source, **variables):
 	try:
 		t3.renders(**variables)
 	except Exception, exc:
-		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
+		assert re.search(msg, "{0.__class__.__module__}.{0.__class__.__name__}: {0}".format(exc)) is not None
 	else:
 		py.test.fail("Didn't raise exception")
 	assert s1 == s2 == s3
@@ -192,6 +192,15 @@ def test_date():
 	check('yes', u'<?if 2000-02-29T12:34:56.987654?>yes<?else?>no<?end if?>')
 
 
+def test_color():
+	check('255,255,255,255', u'<?code c = #fff?><?print c[0]?>,<?print c[1]?>,<?print c[2]?>,<?print c[3]?>')
+	check('255,255,255,255', u'<?code c = #ffffff?><?print c[0]?>,<?print c[1]?>,<?print c[2]?>,<?print c[3]?>')
+	check('18,52,86,255', u'<?code c = #123456?><?print c[0]?>,<?print c[1]?>,<?print c[2]?>,<?print c[3]?>')
+	check('17,34,51,68', u'<?code c = #1234?><?print c[0]?>,<?print c[1]?>,<?print c[2]?>,<?print c[3]?>')
+	check('18,52,86,120', u'<?code c = #12345678?><?print c[0]?>,<?print c[1]?>,<?print c[2]?>,<?print c[3]?>')
+	check('yes', u'<?if #fff?>yes<?else?>no<?end if?>')
+
+
 def test_list():
 	check('', u'<?for item in []?><?print item?>;<?end for?>')
 	check('1;', u'<?for item in [1]?><?print item?>;<?end for?>')
@@ -485,11 +494,11 @@ def test_nested():
 	sv = u"x"
 	n = 4
 	for i in xrange(8): # when using 10 compiling the variable will run out of registers
-		sc = u"(%s)+(%s)" % (sc, sc)
-		sv = u"(%s)+(%s)" % (sv, sv)
-		n = n+n
-	check(str(n), u'<?print %s?>' % sc)
-	check(str(n), u'<?code x=4?><?print %s?>' % sv)
+		sc = u"({})+({})".format(sc, sc)
+		sv = u"({})+({})".format(sv, sv)
+		n = n + n
+	check(str(n), u'<?print {}?>'.format(sc))
+	check(str(n), u'<?code x=4?><?print {}?>'.format(sv))
 
 
 def test_precedence():
@@ -510,11 +519,11 @@ def test_bracket():
 	sc = u"4"
 	sv = u"x"
 	for i in xrange(10):
-		sc = u"(%s)" % sc
-		sv = u"(%s)" % sv
+		sc = u"({})".format(sc)
+		sv = u"({})".format(sv)
 
-	check("4", u'<?print %s?>' % sc)
-	check("4", u'<?code x=4?><?print %s?>' % sv)
+	check("4", u'<?print {}?>'.format(sc))
+	check("4", u'<?code x=4?><?print {}?>'.format(sv))
 
 
 def test_function_now():
@@ -524,12 +533,39 @@ def test_function_now():
 	checkle(now, u"<?print now()?>")
 
 
+def test_function_utcnow():
+	checkrunerror("function u?'utcnow' unknown", u"<?print utcnow(1)?>")
+	checkrunerror("function u?'utcnow' unknown", u"<?print utcnow(1, 2)?>")
+	utcnow = unicode(datetime.datetime.utcnow())
+	checkle(utcnow, u"<?print utcnow()?>")
+
+
 def test_function_vars():
 	checkrunerror("function u?'vars' unknown", "<?print vars(1)?>")
 	checkrunerror("function u?'vars' unknown", "<?print vars(1, 2)?>")
 	check("yes", u"<?if 'spam' in vars()?>yes<?else?>no<?end if?>", spam="eggs")
 
 
+def test_function_random():
+	checkrunerror("function u?'random' unknown", "<?print random(1)?>")
+	checkrunerror("function u?'random' unknown", "<?print random(1, 2)?>")
+	check("ok", u"<?code r = random()?><?if r>=0 and r<1?>ok<?else?>fail<?end if?>")
+
+
+def test_function_randrange():
+	checkrunerror("function u?'randrange' unknown", "<?print randrange()?>")
+	check("ok", u"<?code r = randrange(4)?><?if r>=0 and r<4?>ok<?else?>fail<?end if?>")
+	check("ok", u"<?code r = randrange(17, 23)?><?if r>=17 and r<23?>ok<?else?>fail<?end if?>")
+	check("ok", u"<?code r = randrange(17, 23, 2)?><?if r>=17 and r<23 and r%2?>ok<?else?>fail<?end if?>")
+
+
+def test_function_randchoice():
+	checkrunerror("function u?'randrange' unknown", "<?print randrange()?>")
+	check("ok", u"<?code r = randchoice('abc')?><?if r in 'abc'?>ok<?else?>fail<?end if?>")
+	check("ok", u"<?code s = [17, 23, 42]?><?code r = randchoice(s)?><?if r in s?>ok<?else?>fail<?end if?>")
+	check("ok", u"<?code s = #12345678?><?code sl = [0x12, 0x34, 0x56, 0x78]?><?code r = randchoice(s)?><?if r in sl?>ok<?else?>fail<?end if?>")
+
+
 def test_function_xmlescape():
 	checkrunerror("function u?'xmlescape' unknown", u"<?print xmlescape()?>")
 	checkrunerror("function u?'xmlescape' unknown", u"<?print xmlescape(1, 2)?>")
@@ -576,16 +612,28 @@ def test_function_str():
 
 def test_function_int():
 	checkrunerror("function u?'int' unknown", u"<?print int()?>")
-	checkrunerror("function u?'int' unknown", u"<?print int(1, 2)?>")
+	checkrunerror("function u?'int' unknown", u"<?print int(1, 2, 3)?>")
 	checkrunerror("int\\(\\) argument must be a string or a number, not 'NoneType'", u"<?print int(data)?>", data=None)
 	check("1", u"<?print int(data)?>", data=True)
 	check("0", u"<?print int(data)?>", data=False)
 	check("42", u"<?print int(data)?>", data=42)
 	check("4", u"<?print int(data)?>", data=4.2)
 	check("42", u"<?print int(data)?>", data="42")
+	check("66", u"<?print int(data, 16)?>", data="42")
 	checkrunerror("invalid literal for int\\(\\) with base 10: 'foo'", u"<?print int(data)?>", data="foo")
 
 
+def test_function_float():
+	checkrunerror("function u?'float' unknown", u"<?print float()?>")
+	checkrunerror("function u?'float' unknown", u"<?print float(1, 2, 3)?>")
+	checkrunerror("float\\(\\) argument must be a string or a number", u"<?print float(data)?>", data=None)
+	check("1.0", u"<?print float(data)?>", data=True)
+	check("0.0", u"<?print float(data)?>", data=False)
+	check("42.0", u"<?print float(data)?>", data=42)
+	check("4.2", u"<?print float(data)?>", data=4.2)
+	check("42.0", u"<?print float(data)?>", data="42")
+
+
 def test_function_len():
 	checkrunerror("function u?'len' unknown", u"<?print len()?>")
 	checkrunerror("function u?'len' unknown", u"<?print len(1, 2)?>")
@@ -628,6 +676,7 @@ def test_function_isnone():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isbool():
@@ -645,6 +694,7 @@ def test_function_isbool():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isint():
@@ -662,6 +712,7 @@ def test_function_isint():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isfloat():
@@ -679,6 +730,7 @@ def test_function_isfloat():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isstr():
@@ -696,6 +748,7 @@ def test_function_isstr():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isdate():
@@ -713,6 +766,7 @@ def test_function_isdate():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_islist():
@@ -730,6 +784,7 @@ def test_function_islist():
 	check("True", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isdict():
@@ -747,6 +802,7 @@ def test_function_isdict():
 	check("False", code, data=[])
 	check("True", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_istemplate():
@@ -764,6 +820,25 @@ def test_function_istemplate():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("True", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
+
+
+def test_function_iscolor():
+	checkrunerror("function u?'iscolor' unknown", u"<?print iscolor()?>")
+	checkrunerror("function u?'iscolor' unknown", u"<?print iscolor(1, 2)?>")
+	code = u"<?print iscolor(data)?>"
+	check("False", code, data=None)
+	check("False", code, data=True)
+	check("False", code, data=False)
+	check("False", code, data=42)
+	check("False", code, data=4.2)
+	check("False", code, data="foo")
+	check("False", code, data=datetime.datetime.now())
+	check("False", code, data=())
+	check("False", code, data=[])
+	check("False", code, data={})
+	check("False", code, data=ul4c.compile(u""))
+	check("True", code, data=color.red)
 
 
 def test_function_get():
@@ -834,6 +909,15 @@ def test_function_bin():
 	check("-0b1111", code, data=-15)
 
 
+def test_function_abs():
+	checkrunerror("function u?'abs' unknown", u"<?print abs()?>")
+	checkrunerror("function u?'abs' unknown", u"<?print abs(1, 2)?>")
+	code = u"<?print abs(data)?>"
+	check("0", code, data=0)
+	check("42", code, data=42)
+	check("42", code, data=-42)
+
+
 def test_function_sorted():
 	checkrunerror("function u?'sorted' unknown", u"<?print sorted()?>")
 	checkrunerror("function u?'sorted' unknown", u"<?print sorted(1, 2)?>")
@@ -895,9 +979,19 @@ def test_function_type():
 	check("list", code, x=[1, 2])
 	check("dict", code, x={1: 2})
 	check("template", code, x=ul4c.compile(""))
+	check("color", code, x=color.red)
 	check("", code, x=1j)
 
 
+def test_function_reversed():
+	checkrunerror("function u?'reversed' unknown", u"<?print reversed()?>")
+	checkrunerror("function u?'reversed' unknown", u"<?print reversed(1, 2)?>")
+	code = u"<?for i in reversed(x)?>(<?print i?>)<?end for?>"
+	check("(3)(2)(1)", code, x="123")
+	check("(3)(2)(1)", code, x=[1, 2, 3])
+	check("(3)(2)(1)", code, x=(1, 2, 3))
+
+
 def test_method_upper():
 	check("GURK", u"<?print 'gurk'.upper()?>")
 
@@ -953,10 +1047,20 @@ def test_method_replace():
 	check('goork', ur"<?print 'gurk'.replace('u', 'oo')?>")
 
 
+def test_method_render():
+	t = ul4c.compile(u'(<?print data?>)')
+	check('(GURK)', u"<?print t.render(data='gurk').upper()?>", t=t)
+	check('(GURK)', u"<?print t.render(**{'data': 'gurk'}).upper()?>", t=t)
+
+	t = ul4c.compile(u'(gurk)')
+	check('(GURK)', u"<?print t.render().upper()?>", t=t)
+
+
 def test_method_format():
 	now = datetime.datetime.now()
 	format = "%Y-%m-%d %H:%M:%S"
-	check(now.strftime(format), ur"<?print data.format('%s')?>" % format, data=now)
+	check(now.strftime(format), u"<?print data.format('{}')?>".format(format), data=now)
+	check('987654', u'<?print 2000-02-29T12:34:56.987654.format("%f")?>')
 
 
 def test_method_isoformat():
@@ -964,6 +1068,11 @@ def test_method_isoformat():
 	check(now.isoformat(), ur"<?print data.isoformat()?>", data=now)
 
 
+def test_method_isoformat():
+	now = datetime.datetime(2010, 02, 22, 12, 34, 56)
+	check('Mon, 22 Feb 2010 12:34:56 GMT', ur"<?print data.mimeformat()?>", data=now)
+
+
 def test_method_get():
 	check("42", u"<?print {}.get('foo', 42)?>")
 	check("17", u"<?print {'foo': 17}.get('foo', 42)?>")
@@ -971,6 +1080,105 @@ def test_method_get():
 	check("17", u"<?print {'foo': 17}.get('foo')?>")
 
 
+def test_method_r_g_b_a():
+	check('0x11', u'<?code c = #123?><?print hex(c.r())?>')
+	check('0x22', u'<?code c = #123?><?print hex(c.g())?>')
+	check('0x33', u'<?code c = #123?><?print hex(c.b())?>')
+	check('0xff', u'<?code c = #123?><?print hex(c.a())?>')
+
+
+def test_method_hls():
+	check('0', u'<?code c = #fff?><?print int(c.hls()[0])?>')
+	check('1', u'<?code c = #fff?><?print int(c.hls()[1])?>')
+	check('0', u'<?code c = #fff?><?print int(c.hls()[2])?>')
+
+
+def test_method_hlsa():
+	check('0', u'<?code c = #fff?><?print int(c.hlsa()[0])?>')
+	check('1', u'<?code c = #fff?><?print int(c.hlsa()[1])?>')
+	check('0', u'<?code c = #fff?><?print int(c.hlsa()[2])?>')
+	check('1', u'<?code c = #fff?><?print int(c.hlsa()[3])?>')
+
+
+def test_method_hsv():
+	check('0', u'<?code c = #fff?><?print int(c.hsv()[0])?>')
+	check('0', u'<?code c = #fff?><?print int(c.hsv()[1])?>')
+	check('1', u'<?code c = #fff?><?print int(c.hsv()[2])?>')
+
+
+def test_method_hsva():
+	check('0', u'<?code c = #fff?><?print int(c.hsva()[0])?>')
+	check('0', u'<?code c = #fff?><?print int(c.hsva()[1])?>')
+	check('1', u'<?code c = #fff?><?print int(c.hsva()[2])?>')
+	check('1', u'<?code c = #fff?><?print int(c.hsva()[3])?>')
+
+
+def test_method_lum():
+	check('True', u'<?print #fff.lum() == 1?>')
+
+
+def test_method_withlum():
+	check('#fff', u'<?print #000.withlum(1)?>')
+
+
+def test_method_witha():
+	check('#0063a82a', u'<?print repr(#0063a8.witha(42))?>')
+
+
+def test_method_join():
+	check('1,2,3,4', u'<?print ",".join("1234")?>')
+	check('1,2,3,4', u'<?print ",".join([1, 2, 3, 4])?>')
+
+
+def test_method_day():
+	check('12', u'<?print 2010-05-12T.day()?>')
+	check('12', u'<?print d.day()?>', d=datetime.date(2010, 5, 12))
+
+
+def test_method_month():
+	check('5', u'<?print 2010-05-12T.month()?>')
+	check('5', u'<?print d.month()?>', d=datetime.date(2010, 5, 12))
+
+
+def test_method_year():
+	check('5', u'<?print 2010-05-12T.month()?>')
+	check('5', u'<?print d.month()?>', d=datetime.date(2010, 5, 12))
+
+
+def test_method_hour():
+	check('16', u'<?print 2010-05-12T16:47:56.hour()?>')
+	check('16', u'<?print d.hour()?>', d=datetime.datetime(2010, 5, 12, 16, 47, 56))
+
+
+def test_method_minute():
+	check('47', u'<?print 2010-05-12T16:47:56.minute()?>')
+	check('47', u'<?print d.minute()?>', d=datetime.datetime(2010, 5, 12, 16, 47, 56))
+
+
+def test_method_second():
+	check('56', u'<?print 2010-05-12T16:47:56.second()?>')
+	check('56', u'<?print d.second()?>', d=datetime.datetime(2010, 5, 12, 16, 47, 56))
+
+
+def test_method_microsecond():
+	check('123456', u'<?print 2010-05-12T16:47:56.123456.microsecond()?>')
+	check('123456', u'<?print d.microsecond()?>', d=datetime.datetime(2010, 5, 12, 16, 47, 56, 123456))
+
+
+def test_method_weekday():
+	check('2', u'<?print 2010-05-12T.weekday()?>')
+	check('2', u'<?print d.weekday()?>', d=datetime.date(2010, 5, 12))
+
+
+def test_method_yearday():
+	check('1', u'<?print 2010-01-01T.yearday()?>')
+	check('365', u'<?print 2010-12-31T.yearday()?>')
+	check('132', u'<?print 2010-05-12T.yearday()?>')
+	check('132', u'<?print 2010-05-12T16:47:56.yearday()?>')
+	check('132', u'<?print d.yearday()?>', d=datetime.date(2010, 5, 12))
+	check('132', u'<?print d.yearday()?>', d=datetime.datetime(2010, 5, 12, 16, 47, 56))
+
+
 def test_render():
 	t = ul4c.compile(u'<?print prefix?><?print data?><?print suffix?>')
 	check('(f)(o)(o)', u'<?for c in data?><?render t(data=c, prefix="(", suffix=")")?><?end for?>', t=t, data='foo')
@@ -982,6 +1190,10 @@ def test_render_var():
 	check('42,43,42', u'<?print x?>,<?render t(x=x)?>,<?print x?>', t=t, x=42)
 
 
+def test_def():
+	check('foo', u'<?def lower?><?print x.lower()?><?end def?><?print lower.render(x="FOO")?>')
+
+
 def test_parse():
 	check('42', u'<?print data.Noner?>', data=dict(Noner=42))
 
@@ -996,3 +1208,81 @@ def test_nested_exceptions():
 
 def test_note():
 	check("foo", u"f<?note This is?>o<?note a comment?>o")
+
+
+def universaltemplate():
+	return ul4c.compile("""
+		text
+		<?code x = 'gurk'?>
+		<?code x = 42?>
+		<?code x = 4.2?>
+		<?code x = None?>
+		<?code x = False?>
+		<?code x = True?>
+		<?code x = 2009-01-04T?>
+		<?code x = #0063a8?>
+		<?code x = [42]?>
+		<?code x = {"fortytwo": 42}?>
+		<?code x = {**{"fortytwo": 42}}?>
+		<?code x = y?>
+		<?code x += 42?>
+		<?code x -= 42?>
+		<?code x *= 42?>
+		<?code x /= 42?>
+		<?code x //= 42?>
+		<?code x %= 42?>
+		<?code del x?>
+		<?print x.gurk?>
+		<?print x["gurk"]?>
+		<?print x[1:2]?>
+		<?print x[1:]?>
+		<?print x[:2]?>
+		<?printx x?>
+		<?for x in "12"?><?print x?><?break?><?continue?><?end for?>
+		<?print not x?>
+		<?print -x?>
+		<?print x in y?>
+		<?print x not in y?>
+		<?print x==y?>
+		<?print x!=y?>
+		<?print x<y?>
+		<?print x<=y?>
+		<?print x>y?>
+		<?print x>=y?>
+		<?print x+y?>
+		<?print x*y?>
+		<?print x/y?>
+		<?print x//y?>
+		<?print x and y?>
+		<?print x or y?>
+		<?print x % y?>
+		<?print now()?>
+		<?print repr(1)?>
+		<?print range(1, 2)?>
+		<?print range(1, 2, 3)?>
+		<?print rgb(1, 2, 3, 4)?>
+		<?print x.r()?>
+		<?print x.find(1)?>
+		<?print x.find(1, 2)?>
+		<?print x.find(1, 2, 3)?>
+		<?if x?>gurk<?elif y?>hurz<?else?>hinz<?end if?>
+		<?render x(a=1, b=2)?>
+		<?def x?>foo<?end def?>
+		<?render x()?>
+	""")
+
+
+def test_strtemplate():
+	t = universaltemplate()
+	str(t)
+
+
+def test_pythonsource():
+	t = universaltemplate()
+	t.pythonsource()
+	t.pythonsource("template")
+
+
+def test_pythonfunction():
+	t = universaltemplate()
+	t.pythonfunction()
